from worm2d import Engine, Viewport
import pygame, math
from utility import*
import shipbase, enemy_ship, data,formation,background, enemy_platform, enemy_cruiser
from vector import Vector


objects = [["goodguyship", "ship",35],
           ["maggot", "ship",25],
           ["fighter", "ship",35],
           ["heavy fighter", "ship",40],
           ["imobilizare", "ship",70],
           ["transport", "ship",200],
           ["fuel tanker", "ship",250],
           ["science facility", "ship",500],
           ["platform_main", "platform", 500],
           ["cruiser", "platform", 2000]]


class Background(object):
    def __init__(self, layers):
        self.layers = layers
    def draw(self, camera, engine, data):
        for b in self.layers:
            w2 = 0.2*camera[3]
            w = (5.0*100.0)*b[2]
            h = (5.0*75.0)*b[2]
            x = camera[0] - (b[1]*camera[0])%w
            y = camera[1] - (b[1]*camera[1])%h

            z = (1.0-b[1])*(-80)

            sr = 100*camera[3]

            engine.DrawQuad(x,y,w2,w2,1,1,1,1,0,z=-80)

        
            for bx in range(int(sr/w)+1):
                tx = ((bx+1)*w)
                engine.DrawQuad(x+tx,y,w2,w2,1,1,1,1,0,z=-80)
                engine.DrawQuad(x-tx,y,w2,w2,1,1,1,1,0,z=-80)

            for by in range(int(sr/h)+1):
                ty = ((by+1)*h)
                engine.DrawQuad(x,y+ty,w2,w2,1,1,1,1,0,z=-80)
                engine.DrawQuad(x,y-ty,w2,w2,1,1,1,1,0,z=-80)


class LevelObject(object):
    def __init__(self,x,y,dx,dy,obj):
        self.x = x
        self.y = y

        self.dir = Vector([dx,dy])

        self.type = int(obj)

        self.options = {}

    def draw(self,game):
        global objects
        a = Vector(self.dir).angle+90
        game.engine.DrawImage(game.data.images[objects[self.type][0]],self.x,self.y,objects[self.type][2],objects[self.type][2],1,a)
    def GetCr(self):
        return objects[self.type][2]/2

    def get_string(self):
        return "SHIP " + objects[self.type][0].replace(" ","+") + " " + str(self.x) + " " + str(self.y) + " " + str(self.dir.x) + " " + str(self.dir.y) + "\n"

    


class Game(object):
    def __init__(self, filename, engine):

        self.filename = filename

        pygame.event.set_grab(True)
        pygame.mouse.set_visible(False)
        pygame.mouse.set_pos([0, 0])

        
        self.engine = engine
        self.view = Viewport(0,0,1,1, 100.0, 75.0)

        self.data = data.DataManager()
        self.data.load(self.engine)

        
        self.objects = []

	self.cam = [0,0,0, 20.0]


	self.bg = Background([("grid",1.0,1.0)])
	#self.bg = background.Background([])

	self.ticks = engine.GetTicks()

	self.camspeed = 200.0

	self.selected = 1

	self.dir = [0,1]

	self.mx = 0
	self.my = 0

    def save(self):
        f = open(self.filename, "w")

        for i in self.objects:
            f.write(i.get_string())

        f.close()


    def load(self):
        f = open(self.filename, "r")

        l = f.readlines()

        for i in l:
            j = i.strip("\n").split(" ")

            if j[0] == "SHIP":
                name = j[1].replace("+"," ")
                x,y, = float(j[2]),float(j[3])
                dx,dy = float(j[4]),float(j[5])
                k = 0
                for n in range(len(objects)):
                    if objects[n][0] == name:
                        k = n
                self.add_object(x,y,dx,dy,k)            

        f.close()

        
    
    def ParseEvents(self):
        for event in pygame.event.get():
            if event.type == pygame.KEYDOWN:
                if event.key == pygame.K_ESCAPE:
                    return -1
                if event.key == pygame.K_q:
                    self.selected += 1
                    if self.selected > len(objects)-1:
                        self.selected = 0
                if event.key == pygame.K_e:
                    self.selected -= 1
                    if self.selected < 0:
                        self.selected = len(objects)-1
                if event.key == pygame.K_UP:
                    if self.dir[1] == 1:
                        self.dir[0] = 0
                    else:
                        self.dir[1] = 1
                if event.key == pygame.K_DOWN:
                    if self.dir[1] == -1:
                        self.dir[0] = 0
                    else:
                        self.dir[1] = -1
                if event.key == pygame.K_LEFT:
                    if self.dir[0] == 1:
                        self.dir[1] = 0
                    else:
                        self.dir[0] = 1
                if event.key == pygame.K_RIGHT:
                    if self.dir[0] == -1:
                        self.dir[1] = 0
                    else:
                        self.dir[0] = -1
                if event.key == pygame.K_c:
                    if pygame.key.get_pressed()[pygame.K_LCTRL] or pygame.key.get_pressed()[pygame.K_RCTRL]:
                        self.objects = []
                if event.key == pygame.K_r:
                    if pygame.key.get_pressed()[pygame.K_LCTRL] or pygame.key.get_pressed()[pygame.K_RCTRL]:
                        self.save()
                if event.key == pygame.K_o:
                    if pygame.key.get_pressed()[pygame.K_LCTRL] or pygame.key.get_pressed()[pygame.K_RCTRL]:
                        self.objects = []
                        self.load()
            if event.type == pygame.MOUSEBUTTONDOWN:
                if event.button == 4:
                    self.cam[3] /= 1.2
                elif event.button == 5:
                    self.cam[3] *= 1.2
                elif event.button == 2:
                    self.cam[3] = 10.0
                elif event.button == 1:
                    self.add_object(self.mx,self.my, self.dir[0],self.dir[1],self.selected)
                elif event.button == 3:
                    self.remove_object(self.mx,self.my)
                
    def update(self, delta):
        keys = pygame.key.get_pressed()

        m = 1

        if keys[pygame.K_LCTRL] or keys[pygame.K_RCTRL]:
            m = 20
        
        if keys[pygame.K_w]:
            self.cam[1] -= self.camspeed*delta*m
        if keys[pygame.K_s]:
            self.cam[1] += self.camspeed*delta*m
        if keys[pygame.K_a]:
            self.cam[0] -= self.camspeed*delta*m
        if keys[pygame.K_d]:
            self.cam[0] += self.camspeed*delta*m

        msx,msy = pygame.mouse.get_pos()
        self.mx,self.my = self.engine.MouseCoordsToWorldCoords(msx,msy,self.cam[0],self.cam[1],self.cam[3], self.view)

    def draw_object(self,s, x,y):
        global objects
        a = Vector(self.dir).angle+90
        self.engine.DrawImage(self.data.images[objects[s][0]],x,y,objects[s][2],objects[s][2],1,a)

    def add_object(self,x,y,dx,dy,s):
        self.objects.append(LevelObject(x,y,dx,dy,s))
    def remove_object(self,x,y):
        for i in self.objects:
            if Distance(x,y,i.x,i.y) < i.GetCr():
                self.objects.remove(i)
        

        
    def draw(self):
        self.engine.PreRender(0.2, 0.2, 0.2)
        self.engine.SetView(self.view, self.cam[0],self.cam[1],self.cam[3], angle = self.cam[2])

        self.bg.draw(self.cam, self.engine, self.data)

        for o in self.objects:
            o.draw(self)

        self.draw_object(self.selected, self.mx,self.my)

        self.engine.DrawQuad(0,0,20,20,1,1,1,1,0)

        

        self.engine.SetView(self.view,0,0,1)

        self.engine.DrawString(str(int(self.mx)) + ", " + str(int(self.my)), 0,-36,3,3 ,1,1,1,1,0)

        self.engine.PostRender()
        
    def run(self):
        while self.engine.IsRunning():
            delta = float(self.engine.GetTicks() - self.ticks)/1000.0
            self.ticks = self.engine.GetTicks()
            
            r = self.ParseEvents()
            if r == -1:
                return 0

            r = self.update(delta)
            if r == -1:
                return 0

            self.draw()


def main():
    
    engine = Engine(640,480, 32, False, "Awesome Nine Editor")

    game = Game("data/testlevel.txt", engine)

    game.run()

