from worm2d import Engine, Viewport
import pygame, math,random
from utility import*
from vector import Vector



delta = {9: [ (0,0), (-4,-2), (4, -2), (-7,-5), (7,-5), (-2,-8), (2, -8), (-9,-9),(9,-9)],
         8: [ (0,0), (-2,-2), (2,-2), (-4,-5), (4,-5), (0,  -7), (-6,-7), (6, -7)],
         7: [ (0,0), (-3,-2), (3,-2), (-2,-6), (2,-6), (-6,-6), (6, -6)],
         6: [ (0,0), (-2,-2), (2,-2), (0, -4),(-4,-4), (4, -4)],
         5: [ (0,0), (-2,-2), (2,-2), (-4,-4),(4, -4)],
         4: [ (0,0), (-2,-2), (2,-2), (0, -3)],
         3: [ (0,0), (-2,-2), (2,-2)],
         2: [ (0,0), (-2,-2)],
         1: [ (0,0)]
         }

horiz = {9: [ (0,0), (-3, 0), (3,0), (-6,0), (6,0), (-9,0), (9,0), (-12,0), (12,0)],
         8: [ (0,0), (-3, 0), (3,0), (-6,0), (6,0), (-9,0), (9,0), (-12,0)],
         7: [ (0,0), (-3, 0), (3,0), (-6,0), (6,0), (-9,0), (9,0)],
         6: [ (0,0), (-3, 0), (3,0), (-6,0), (6,0), (-9,0)],
         5: [ (0,0), (-3, 0), (3,0), (-6,0), (6,0)],
         4: [ (0,0), (-3, 0), (3,0), (-6,0)],
         3: [ (0,0), (-3, 0), (3,0)],
         2: [ (0,0), (-3, 0)],
         1: [(0,0)]
         }


class Formation(object):
    def __init__(self, ids, ships,form = "Delta"):
        self.ids = ids
        self.c = (form == "cont")
        self.mode = "delta"
        self.spread = 10.0
        cnt = 0
        for i in ids:
            for s in ships:
                if i == id(s):
                    if cnt == 0:
                        s.isleader = True
                    s.mode = self.mode
                    s.inFormation = True
                else:
                    s.inFormation = True
                
            cnt += 1

        self.set_positions(ships,True)
                    
                        
    def update(self, ships,game,delta):

        if len(self.ids) == 0:
            return -1

        for i in range(len(self.ids)):
            #Ranked first to last
            for s in ships:
                if self.ids[i] == id(s):
                    #s is the current ship, ranked i
                    if i == 0:
                        s.LeaderUpdate(game, delta)
                        
        
        self.set_positions(ships)        

    def set_positions(self,ships,first = False):
        global delta
        leader_pos = Vector([0,0])
        leader_angle = 0
        shooting = False
        prev_pos = Vector([0,0])
        prev_angle = 0
        prev_dir = Vector([0,0])
        for i in range(len(self.ids)):
            #Ranked first to last
            for s in ships:
                if self.ids[i] == id(s):
                    #s is the current ship, ranked i
                    if i == 0:
                        s.isleader = True
                        leader_pos = Vector([s.x,s.y])
                        leader_angle = s.dir.angle
                        shooting = s.isShooting
                        
                        #for 'follow'
                        prev_pos = leader_pos
                        prev_angle = leader_angle
                        prev_dir = s.dir
                        
                    s.mode = self.mode

                    if i != 0:
                        s.isShooting = shooting
                        if self.mode == "delta":
                            x = (delta[len(self.ids)][i][0]) * self.spread
                            y = (delta[len(self.ids)][i][1]) * self.spread
                            spot = Vector([x,y])
                            spot = spot.rotated(leader_angle-90)
                            spot = spot+leader_pos
                            
                            if first == True:
                                s.x, s.y = spot.x,spot.y
                            else:
                                s.point = [spot.x,spot.y]
                                s.angle_to = leader_angle
                        if self.mode == "horiz":
                            x = (horiz[len(self.ids)][i][0]) * self.spread
                            y = (horiz[len(self.ids)][i][1]) * self.spread
                            spot = Vector([x,y])
                            spot = spot.rotated(leader_angle-90)
                            spot = spot+leader_pos
                            
                            if first == True:
                                s.x, s.y = spot.x,spot.y
                            else:
                                s.point = [spot.x,spot.y]
                                s.angle_to = leader_angle

                        if self.mode == "follow":
                            aim_pos = prev_pos - ((leader_pos - Vector([s.x,s.y])).normalised())*self.spread*3
                            aim_angle = (prev_pos - Vector([s.x,s.y])).angle

                            prev_pos = Vector([s.x,s.y])
                            prev_angle = s.angle_to
                            #prev_dir = s.dir

                            s.point = [aim_pos.x,aim_pos.y]
                            s.angle_to = aim_angle

                            
                                
            

    def on_ship_die(self,ship_id,ships):
        for i in self.ids:
            if i == ship_id:
                self.ids.remove(i)
        cnt = 0
        for i in self.ids:
            for s in ships:
                if i == id(s) and cnt == 0:
                    s.isleader = True
                    print id(s)                
            cnt += 1
        

        
        
    def get_event(self,event):
        if self.c:
            if event.type == pygame.KEYDOWN:
                if event.key == pygame.K_q:
                    self.mode = "delta"
                elif event.key == pygame.K_w:
                    self.mode = "horiz"
                elif event.key == pygame.K_e:
                    self.mode = "follow"

